package org.mapsforge.android.maps;


import java.util.ArrayList;
import java.util.Collection;

import android.app.AlertDialog;
import android.content.Context;
import android.graphics.drawable.Drawable;

/**
 * Thread-safe implementation of the {@link ItemizedMarkerOverlay} class using an {@link ArrayList} as
 * internal data structure.
 */
public class ArrayItemizedMarkerOverlay extends ItemizedMarkerOverlay<OverlayMarkerItem> {
	private static final int ARRAY_LIST_INITIAL_CAPACITY = 8;
	private static final String THREAD_NAME = "ArrayItemizedMarkerOverlay";

	protected final Context context;
	protected AlertDialog.Builder dialog;
	protected OverlayItem item;
	protected final ArrayList<OverlayMarkerItem> overlayMarkerItems;

	/**
	 * Constructs a new ArrayItemizedOverlay.
	 * 
	 * @param defaultMarker
	 *            the default marker.
	 * @param context
	 *            the reference to the application context.
	 */
	public ArrayItemizedMarkerOverlay(Drawable defaultMarker, Context context) {
		super(boundCenterBottom(defaultMarker));
		this.context = context;
		this.overlayMarkerItems = new ArrayList<OverlayMarkerItem>(ARRAY_LIST_INITIAL_CAPACITY);
	}

	/**
	 * Adds the given item to the Overlay.
	 * 
	 * @param overlayItem
	 *            the item that should be added to the Overlay.
	 */
	public synchronized void addOverlay(OverlayMarkerItem overlayItem) {
		this.overlayMarkerItems.add(overlayItem);
		populate();
	}

	/**
	 * Adds all items of the given collection to the Overlay.
	 * 
	 * @param c
	 *            collection whose items should be added to the Overlay.
	 */
	public synchronized void addOverlays(Collection<? extends OverlayMarkerItem> c) {
		this.overlayMarkerItems.addAll(c);
		populate();
	}

	/**
	 * Removes all items from the Overlay.
	 */
	public synchronized void clear() {
		this.overlayMarkerItems.clear();
		populate();
	}

	@Override
	public String getThreadName() {
		return THREAD_NAME;
	}

	/**
	 * Removes the given item from the Overlay.
	 * 
	 * @param overlayMarkerItem
	 *            the item that should be removed from the Overlay.
	 */
	public synchronized void removeOverlay(OverlayMarkerItem overlayMarkerItem) {
		this.overlayMarkerItems.remove(overlayMarkerItem);
		populate();
	}

	@Override
	public synchronized int size() {
		return this.overlayMarkerItems.size();
	}

	@Override
	public synchronized OverlayMarkerItem createItem(int i) {
		return this.overlayMarkerItems.get(i);
	}
}